@@ -6,12 +6,12 @@ capabilities.
 
 Public Functions in flanker.addresslib.address module:
 
-    * parse(address, addr_spec_only=False, strict=False)
+    * parse(address, addr_spec_only=False)
 
       Parse a single address or URL. Can parse just the address spec or the
       full mailbox.
 
-    * parse_list(address_list, strict=False)
+    * parse_list(address_list, strict=False, as_tuple=False)
 
       Parse a list of addresses, operates in strict or relaxed modes. Strict
       mode will fail at the first instance of invalid grammar, relaxed modes
@@ -23,11 +23,10 @@ Public Functions in flanker.addresslib.address module:
       address spec. In the case of a valid address returns an EmailAddress
       object, otherwise returns None.
 
-    * validate_list(addr_list, syntax_only=True)
+    * validate_list(addr_list, as_tuple=False)
 
       Validates an address list, and returns a tuple of parsed and unparsed
-      portions. Can parse syntax only mode (no dns, mx check, or custom
-      grammar).
+      portions.
 
 When valid addresses are returned, they are returned as an instance of either
 EmailAddress or UrlAddress in flanker.addresslib.address.
@@ -35,29 +34,35 @@ EmailAddress or UrlAddress in flanker.addresslib.address.
 See the parser.py module for implementation details of the parser.
 '''
 
+import time
 import flanker.addresslib.parser
 import flanker.addresslib.validate
 
 from email.header import Header
 from email.utils import formataddr
 from flanker.utils import is_pure_ascii
+from flanker.utils import metrics_wrapper
 from flanker.mime.message.headers.encoding import encode_string
 from flanker.mime.message.headers.encodedword import mime_to_unicode
 from urlparse import urlparse
 
-def parse(address, addr_spec_only=False):
+@metrics_wrapper()
+def parse(address, addr_spec_only=False, metrics=False):
     '''
     Given an string, returns a scalar object representing a single full
     mailbox (display name and addr-spec), addr-spec, or a url.
 
+    Returns an Address object and optionally metrics on processing
+    time if requested.
+
     Examples:
         >>> address.parse('John Smith <john@smith.com')
         John Smith <john@smith.com>
 
-        >>> print address.parse('John <john@smith.com>', True)
+        >>> print address.parse('John <john@smith.com>', addr_spec_only=True)
         None
 
-        >>> print address.parse('john@smith.com', True)
+        >>> print address.parse('john@smith.com', addr_spec_only=True)
         'john@smith.com'
 
         >>> address.parse('http://host.com/post?q')
@@ -66,20 +71,31 @@ def parse(address, addr_spec_only=False):
         >>> print address.parse('foo')
         None
     '''
+    mtimes = {'parsing': 0}
+
     parser = flanker.addresslib.parser._AddressParser(False)
 
     try:
-        # try to parse the address
+        # addr-spec only
         if addr_spec_only:
-            return parser.address_spec(address)
-        return parser.address(address)
+            bstart = time.time()
+            retval = parser.address_spec(address)
+            mtimes['parsing'] = time.time() - bstart
+            return retval, mtimes
+
+        # full address
+        bstart = time.time()
+        retval = parser.address(address)
+        mtimes['parsing'] = time.time() - bstart
+        return retval, mtimes
 
     # supress any exceptions and return None
     except flanker.addresslib.parser.ParserException:
-        return None
+        return None, mtimes
 
 
-def parse_list(address_list, strict=False, as_tuple=False):
+@metrics_wrapper()
+def parse_list(address_list, strict=False, as_tuple=False, metrics=False):
     '''
     Given an string or list of email addresses and/or urls seperated by a
     delimiter (comma (,) or semi-colon (;)), returns an AddressList object
@@ -89,6 +105,10 @@ def parse_list(address_list, strict=False, as_tuple=False):
     will quit at the first occurrence of error, in relaxed mode the parser
     will attempt to seek to to known valid location and continue parsing.
 
+    The parser can return a list of parsed addresses or a tuple containing
+    the parsed and unparsed portions. The parser also returns the parsing
+    time metrics if requested.
+
     Examples:
         >>> address.parse_list('A <a@b>')
         [A <a@b>]
@@ -105,6 +125,7 @@ def parse_list(address_list, strict=False, as_tuple=False):
         >>> address.parse_list('A <a@b>, D <d@e>, http://localhost')
         [A <a@b>, D <d@e>, http://localhost]
     '''
+    mtimes = {'parsing': 0}
     parser = flanker.addresslib.parser._AddressParser(strict)
 
     # if we have a list, transform it into a string first
@@ -113,28 +134,31 @@ def parse_list(address_list, strict=False, as_tuple=False):
 
     # parse
     try:
+        bstart = time.time()
         if strict:
             p = parser.address_list(address_list)
             u = []
         else:
             p, u = parser.address_list(address_list)
+        mtimes['parsing'] = time.time() - bstart
     except flanker.addresslib.parser.ParserException:
         p, u = (AddressList(), [])
 
     # return as tuple or just parsed addresses
     if as_tuple:
-        return p, u
-    return p
+        return p, u, mtimes
+    return p, mtimes
 
 
-def validate_address(addr_spec):
+@metrics_wrapper()
+def validate_address(addr_spec, metrics=False):
     '''
     Given an addr-spec, runs the pre-parser, the parser, DNS MX checks,
     MX existence checks, and if available, ESP specific grammar for the
     local part.
 
     In the case of a valid address returns an EmailAddress object, otherwise
-    returns None.
+    returns None. If requested, will also return the parsing time metrics.
 
     Examples:
         >>> address.validate_address('john@non-existent-domain.com')
@@ -146,99 +170,121 @@ def validate_address(addr_spec):
         >>> address.validate_address('user.1234@gmail.com')
         user.1234@gmail.com
     '''
+    mtimes = {'parsing': 0, 'mx_lookup': 0,
+        'dns_lookup': 0, 'mx_conn':0 , 'custom_grammar':0}
+
     # sanity check
     if addr_spec is None:
-        return None
+        return None, mtimes
     if not is_pure_ascii(addr_spec):
-        return None
+        return None, mtimes
 
     # preparse address into its parts and perform any ESP specific pre-parsing
     addr_parts = flanker.addresslib.validate.preparse_address(addr_spec)
     if addr_parts is None:
-        return None
+        return None, mtimes
 
     # run parser against address
+    bstart = time.time()
     paddr = parse('@'.join(addr_parts), addr_spec_only=True)
+    mtimes['parsing'] = time.time() - bstart
     if paddr is None:
-        return None
+        return None, mtimes
 
     # lookup if this domain has a mail exchanger
-    has_exchanger, exchanger = \
-        flanker.addresslib.validate.mail_exchanger_lookup(addr_parts[-1])
+    has_exchanger, exchanger, mx_metrics = \
+        flanker.addresslib.validate.mail_exchanger_lookup(addr_parts[-1], metrics=True)
+    mtimes['mx_lookup'] = mx_metrics['mx_lookup']
+    mtimes['dns_lookup'] = mx_metrics['dns_lookup']
+    mtimes['mx_conn'] = mx_metrics['mx_conn']
     if has_exchanger is False:
-        return None
+        return None, mtimes
 
     # lookup custom local-part grammar if it exists
+    bstart = time.time()
     plugin = flanker.addresslib.validate.plugin_for_esp(exchanger)
+    mtimes['custom_grammar'] = time.time() - bstart
     if plugin and plugin.validate(addr_parts[0]) is False:
-        return None
+        return None, mtimes
 
-    return paddr
+    return paddr, mtimes
 
 
-def validate_list(addr_list, syntax_only=True, as_tuple=False):
+@metrics_wrapper()
+def validate_list(addr_list, as_tuple=False, metrics=False):
     '''
     Validates an address list, and returns a tuple of parsed and unparsed
-    portions. Can parse syntax only mode (no dns, mx check, or custom grammar).
+    portions.
 
-    Returns a tuple consisting of the parsed addresses and unparsable protions.
+    Returns results as a list or tuple consisting of the parsed addresses
+    and unparsable protions. If requested, will also return parisng time
+    metrics.
 
     Examples:
-        >>> address.validate_address_list('a@b, c@d')
-        ([a@b, c@d], [])
+        >>> address.validate_address_list('a@mailgun.com, c@mailgun.com')
+        [a@mailgun.com, c@mailgun.com]
 
-        >>> address.validate_address_list('a@b, c@d, @e')
-        ([a@b, c@d], ['@e'])
+        >>> address.validate_address_list('a@mailgun.com, b@example.com')
+        [a@mailgun.com]
 
-        >>> address.validate_address_list('a@mailgun.com, b@example.com', True)
-        ([a@mailgun.com], ['b@example.com'])
+        >>> address.validate_address_list('a@b, c@d, e@example.com', as_tuple=True)
+        ([a@mailgun.com, c@mailgun.com], ['e@example.com'])
     '''
+    mtimes = {'parsing': 0, 'mx_lookup': 0,
+        'dns_lookup': 0, 'mx_conn':0 , 'custom_grammar':0}
+
+
     if addr_list is None:
-        return None
+        return None, mtimes
 
     # parse addresses
+    bstart = time.time()
     parsed_addresses, unparseable = parse_list(addr_list, as_tuple=True)
+    mtimes['parsing'] = time.time() - bstart
 
-    if syntax_only:
-        if as_tuple:
-            return parsed_addresses, unparseable
-        else:
-            return parsed_addresses
-
-    # we are doing syntax checks as well
-    plist = []
+    plist = flanker.addresslib.address.AddressList()
     ulist = []
 
     # make sure parsed list pass dns and esp grammar
     for paddr in parsed_addresses:
 
         # lookup if this domain has a mail exchanger
-        has_exchanger, exchanger = \
-            flanker.addresslib.validate.mail_exchanger_lookup(paddr.hostname)
+        has_exchanger, exchanger, mx_metrics = \
+            flanker.addresslib.validate.mail_exchanger_lookup(paddr.hostname, metrics=True)
+        mtimes['mx_lookup'] += mx_metrics['mx_lookup']
+        mtimes['dns_lookup'] += mx_metrics['dns_lookup']
+        mtimes['mx_conn'] += mx_metrics['mx_conn']
+
         if has_exchanger is False:
             ulist.append(paddr.full_spec())
             continue
 
         # lookup custom local-part grammar if it exists
         plugin = flanker.addresslib.validate.plugin_for_esp(exchanger)
+        bstart = time.time()
         if plugin and plugin.validate(paddr.mailbox) is False:
             ulist.append(paddr.full_spec())
             continue
+        mtimes['custom_grammar'] = time.time() - bstart
 
         plist.append(paddr)
 
     # loop over unparsable list and check if any can be fixed with
     # preparsing cleanup and if so, run full validator
     for unpar in unparseable:
-        paddr = validate_address(unpar)
+        paddr, metrics = validate_address(unpar, metrics=True)
         if paddr:
             plist.append(paddr)
         else:
             ulist.append(unpar)
 
+        # update all the metrics
+        for k, v in metrics.iteritems():
+            metrics[k] += v
+
     if as_tuple:
-        return plist, ulist
-    return plist
+        return plist, ulist, mtimes
+    return plist, mtimes
 
 
 def is_email(string):
@@ -503,7 +549,9 @@ class AddressList(object):
 
     def __eq__(self, other):
         "When comparing ourselves to other lists we must ignore order"
-        return set(self.container) == set(other)
+        if isinstance(other, list):
+            other = parse_list(other)
+        return set(self.container) == set(other.container)
 
     def __str__(self):
         return ''.join(['[', self.full_spec(), ']'])
